# Generated from FULL_LICENSING_INFORMATION.md on 2025-02-01 17:56:53.440199


content_html = '''
<h1 align="center">Full licencing information</h1>

<p>This file contains all the licensing information for software included or referenced by Multigit.</p>

<h2>IDEMIA Logo</h2>

<p>The IDEMIA logo is a trademark of IDEMIA. If you want to distribute a modified version of Multigit, you need to
either get the approval of IDEMIA for using its logo or remove the IDEMIA logo from the modifed version.</p>

<h2>Multigit</h2>

<p>Multigit software and source code is released under Apache 2.0 License:</p>

<pre><code><span style="background-color:#f5f5f5;">                                 Apache License
                           Version 2.0, January 2004
                        http://www.apache.org/licenses/

TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION

1. Definitions.

   "License" shall mean the terms and conditions for use, reproduction,
   and distribution as defined by Sections 1 through 9 of this document.

   "Licensor" shall mean the copyright owner or entity authorized by
   the copyright owner that is granting the License.

   "Legal Entity" shall mean the union of the acting entity and all
   other entities that control, are controlled by, or are under common
   control with that entity. For the purposes of this definition,
   "control" means (i) the power, direct or indirect, to cause the
   direction or management of such entity, whether by contract or
   otherwise, or (ii) ownership of fifty percent (50%) or more of the
   outstanding shares, or (iii) beneficial ownership of such entity.

   "You" (or "Your") shall mean an individual or Legal Entity
   exercising permissions granted by this License.

   "Source" form shall mean the preferred form for making modifications,
   including but not limited to software source code, documentation
   source, and configuration files.

   "Object" form shall mean any form resulting from mechanical
   transformation or translation of a Source form, including but
   not limited to compiled object code, generated documentation,
   and conversions to other media types.

   "Work" shall mean the work of authorship, whether in Source or
   Object form, made available under the License, as indicated by a
   copyright notice that is included in or attached to the work
   (an example is provided in the Appendix below).

   "Derivative Works" shall mean any work, whether in Source or Object
   form, that is based on (or derived from) the Work and for which the
   editorial revisions, annotations, elaborations, or other modifications
   represent, as a whole, an original work of authorship. For the purposes
   of this License, Derivative Works shall not include works that remain
   separable from, or merely link (or bind by name) to the interfaces of,
   the Work and Derivative Works thereof.

   "Contribution" shall mean any work of authorship, including
   the original version of the Work and any modifications or additions
   to that Work or Derivative Works thereof, that is intentionally
   submitted to Licensor for inclusion in the Work by the copyright owner
   or by an individual or Legal Entity authorized to submit on behalf of
   the copyright owner. For the purposes of this definition, "submitted"
   means any form of electronic, verbal, or written communication sent
   to the Licensor or its representatives, including but not limited to
   communication on electronic mailing lists, source code control systems,
   and issue tracking systems that are managed by, or on behalf of, the
   Licensor for the purpose of discussing and improving the Work, but
   excluding communication that is conspicuously marked or otherwise
   designated in writing by the copyright owner as "Not a Contribution."

   "Contributor" shall mean Licensor and any individual or Legal Entity
   on behalf of whom a Contribution has been received by Licensor and
   subsequently incorporated within the Work.

2. Grant of Copyright License. Subject to the terms and conditions of
   this License, each Contributor hereby grants to You a perpetual,
   worldwide, non-exclusive, no-charge, royalty-free, irrevocable
   copyright license to reproduce, prepare Derivative Works of,
   publicly display, publicly perform, sublicense, and distribute the
   Work and such Derivative Works in Source or Object form.

3. Grant of Patent License. Subject to the terms and conditions of
   this License, each Contributor hereby grants to You a perpetual,
   worldwide, non-exclusive, no-charge, royalty-free, irrevocable
   (except as stated in this section) patent license to make, have made,
   use, offer to sell, sell, import, and otherwise transfer the Work,
   where such license applies only to those patent claims licensable
   by such Contributor that are necessarily infringed by their
   Contribution(s) alone or by combination of their Contribution(s)
   with the Work to which such Contribution(s) was submitted. If You
   institute patent litigation against any entity (including a
   cross-claim or counterclaim in a lawsuit) alleging that the Work
   or a Contribution incorporated within the Work constitutes direct
   or contributory patent infringement, then any patent licenses
   granted to You under this License for that Work shall terminate
   as of the date such litigation is filed.

4. Redistribution. You may reproduce and distribute copies of the
   Work or Derivative Works thereof in any medium, with or without
   modifications, and in Source or Object form, provided that You
   meet the following conditions:

   (a) You must give any other recipients of the Work or
   Derivative Works a copy of this License; and

   (b) You must cause any modified files to carry prominent notices
   stating that You changed the files; and

   (c) You must retain, in the Source form of any Derivative Works
   that You distribute, all copyright, patent, trademark, and
   attribution notices from the Source form of the Work,
   excluding those notices that do not pertain to any part of
   the Derivative Works; and

   (d) If the Work includes a "NOTICE" text file as part of its
   distribution, then any Derivative Works that You distribute must
   include a readable copy of the attribution notices contained
   within such NOTICE file, excluding those notices that do not
   pertain to any part of the Derivative Works, in at least one
   of the following places: within a NOTICE text file distributed
   as part of the Derivative Works; within the Source form or
   documentation, if provided along with the Derivative Works; or,
   within a display generated by the Derivative Works, if and
   wherever such third-party notices normally appear. The contents
   of the NOTICE file are for informational purposes only and
   do not modify the License. You may add Your own attribution
   notices within Derivative Works that You distribute, alongside
   or as an addendum to the NOTICE text from the Work, provided
   that such additional attribution notices cannot be construed
   as modifying the License.

   You may add Your own copyright statement to Your modifications and
   may provide additional or different license terms and conditions
   for use, reproduction, or distribution of Your modifications, or
   for any such Derivative Works as a whole, provided Your use,
   reproduction, and distribution of the Work otherwise complies with
   the conditions stated in this License.

5. Submission of Contributions. Unless You explicitly state otherwise,
   any Contribution intentionally submitted for inclusion in the Work
   by You to the Licensor shall be under the terms and conditions of
   this License, without any additional terms or conditions.
   Notwithstanding the above, nothing herein shall supersede or modify
   the terms of any separate license agreement you may have executed
   with Licensor regarding such Contributions.

6. Trademarks. This License does not grant permission to use the trade
   names, trademarks, service marks, or product names of the Licensor,
   except as required for reasonable and customary use in describing the
   origin of the Work and reproducing the content of the NOTICE file.

7. Disclaimer of Warranty. Unless required by applicable law or
   agreed to in writing, Licensor provides the Work (and each
   Contributor provides its Contributions) on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied, including, without limitation, any warranties or conditions
   of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
   PARTICULAR PURPOSE. You are solely responsible for determining the
   appropriateness of using or redistributing the Work and assume any
   risks associated with Your exercise of permissions under this License.

8. Limitation of Liability. In no event and under no legal theory,
   whether in tort (including negligence), contract, or otherwise,
   unless required by applicable law (such as deliberate and grossly
   negligent acts) or agreed to in writing, shall any Contributor be
   liable to You for damages, including any direct, indirect, special,
   incidental, or consequential damages of any character arising as a
   result of this License or out of the use or inability to use the
   Work (including but not limited to damages for loss of goodwill,
   work stoppage, computer failure or malfunction, or any and all
   other commercial damages or losses), even if such Contributor
   has been advised of the possibility of such damages.

9. Accepting Warranty or Additional Liability. While redistributing
   the Work or Derivative Works thereof, You may choose to offer,
   and charge a fee for, acceptance of support, warranty, indemnity,
   or other liability obligations and/or rights consistent with this
   License. However, in accepting such obligations, You may act only
   on Your own behalf and on Your sole responsibility, not on behalf
   of any other Contributor, and only if You agree to indemnify,
   defend, and hold each Contributor harmless for any liability
   incurred by, or claims asserted against, such Contributor by reason
   of your accepting any such warranty or additional liability.

END OF TERMS AND CONDITIONS
</span></code></pre>

<h2>Python</h2>

<p>Python software is licensed under the PSF License Agreement.</p>

<pre><code><span style="background-color:#f5f5f5;">PSF LICENSE AGREEMENT FOR PYTHON 3.8.10

1. This LICENSE AGREEMENT is between the Python Software Foundation ("PSF"), and
   the Individual or Organization ("Licensee") accessing and otherwise using Python
   3.8.10 software in source or binary form and its associated documentation.

2. Subject to the terms and conditions of this License Agreement, PSF hereby
   grants Licensee a nonexclusive, royalty-free, world-wide license to reproduce,
   analyze, test, perform and/or display publicly, prepare derivative works,
   distribute, and otherwise use Python 3.8.10 alone or in any derivative
   version, provided, however, that PSF's License Agreement and PSF's notice of
   copyright, i.e., "Copyright (c) 2001-2023 Python Software Foundation; All Rights
   Reserved" are retained in Python 3.8.10 alone or in any derivative version
   prepared by Licensee.

3. In the event Licensee prepares a derivative work that is based on or
   incorporates Python 3.8.10 or any part thereof, and wants to make the
   derivative work available to others as provided herein, then Licensee hereby
   agrees to include in any such work a brief summary of the changes made to Python
   3.8.10.

4. PSF is making Python 3.8.10 available to Licensee on an "AS IS" basis.
   PSF MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.  BY WAY OF
   EXAMPLE, BUT NOT LIMITATION, PSF MAKES NO AND DISCLAIMS ANY REPRESENTATION OR
   WARRANTY OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE
   USE OF PYTHON 3.8.10 WILL NOT INFRINGE ANY THIRD PARTY RIGHTS.

5. PSF SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF PYTHON 3.8.10
   FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS A RESULT OF
   MODIFYING, DISTRIBUTING, OR OTHERWISE USING PYTHON 3.8.10, OR ANY DERIVATIVE
   THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

6. This License Agreement will automatically terminate upon a material breach of
   its terms and conditions.

7. Nothing in this License Agreement shall be deemed to create any relationship
   of agency, partnership, or joint venture between PSF and Licensee.  This License
   Agreement does not grant permission to use PSF trademarks or trade name in a
   trademark sense to endorse or promote products or services of Licensee, or any
   third party.

8. By copying, installing or otherwise using Python 3.8.10, Licensee agrees
   to be bound by the terms and conditions of this License Agreement.
</span></code></pre>

<h2>PySide2 / Qt5 for Python</h2>

<p>See <a href="https://doc.qt.io/qt-5/licenses-used-in-qt.html">Qt's detailed licensing information</a> for software
used by Qt5.</p>

<p>Qt/PySide2 is available under LGPLv3.</p>

<pre><code><span style="background-color:#f5f5f5;">             GNU LESSER GENERAL PUBLIC LICENSE
                   Version 3, 29 June 2007

 Copyright (C) 2007 Free Software Foundation, Inc. &lt;https://fsf.org/&gt;
 Everyone is permitted to copy and distribute verbatim copies
 of this license document, but changing it is not allowed.


  This version of the GNU Lesser General Public License incorporates
the terms and conditions of version 3 of the GNU General Public
License, supplemented by the additional permissions listed below.

  0. Additional Definitions.

  As used herein, "this License" refers to version 3 of the GNU Lesser
General Public License, and the "GNU GPL" refers to version 3 of the GNU
General Public License.

  "The Library" refers to a covered work governed by this License,
other than an Application or a Combined Work as defined below.

  An "Application" is any work that makes use of an interface provided
by the Library, but which is not otherwise based on the Library.
Defining a subclass of a class defined by the Library is deemed a mode
of using an interface provided by the Library.

  A "Combined Work" is a work produced by combining or linking an
Application with the Library.  The particular version of the Library
with which the Combined Work was made is also called the "Linked
Version".

  The "Minimal Corresponding Source" for a Combined Work means the
Corresponding Source for the Combined Work, excluding any source code
for portions of the Combined Work that, considered in isolation, are
based on the Application, and not on the Linked Version.

  The "Corresponding Application Code" for a Combined Work means the
object code and/or source code for the Application, including any data
and utility programs needed for reproducing the Combined Work from the
Application, but excluding the System Libraries of the Combined Work.

  1. Exception to Section 3 of the GNU GPL.

  You may convey a covered work under sections 3 and 4 of this License
without being bound by section 3 of the GNU GPL.

  2. Conveying Modified Versions.

  If you modify a copy of the Library, and, in your modifications, a
facility refers to a function or data to be supplied by an Application
that uses the facility (other than as an argument passed when the
facility is invoked), then you may convey a copy of the modified
version:

   a) under this License, provided that you make a good faith effort to
   ensure that, in the event an Application does not supply the
   function or data, the facility still operates, and performs
   whatever part of its purpose remains meaningful, or

   b) under the GNU GPL, with none of the additional permissions of
   this License applicable to that copy.

  3. Object Code Incorporating Material from Library Header Files.

  The object code form of an Application may incorporate material from
a header file that is part of the Library.  You may convey such object
code under terms of your choice, provided that, if the incorporated
material is not limited to numerical parameters, data structure
layouts and accessors, or small macros, inline functions and templates
(ten or fewer lines in length), you do both of the following:

   a) Give prominent notice with each copy of the object code that the
   Library is used in it and that the Library and its use are
   covered by this License.

   b) Accompany the object code with a copy of the GNU GPL and this license
   document.

  4. Combined Works.

  You may convey a Combined Work under terms of your choice that,
taken together, effectively do not restrict modification of the
portions of the Library contained in the Combined Work and reverse
engineering for debugging such modifications, if you also do each of
the following:

   a) Give prominent notice with each copy of the Combined Work that
   the Library is used in it and that the Library and its use are
   covered by this License.

   b) Accompany the Combined Work with a copy of the GNU GPL and this license
   document.

   c) For a Combined Work that displays copyright notices during
   execution, include the copyright notice for the Library among
   these notices, as well as a reference directing the user to the
   copies of the GNU GPL and this license document.

   d) Do one of the following:

       0) Convey the Minimal Corresponding Source under the terms of this
       License, and the Corresponding Application Code in a form
       suitable for, and under terms that permit, the user to
       recombine or relink the Application with a modified version of
       the Linked Version to produce a modified Combined Work, in the
       manner specified by section 6 of the GNU GPL for conveying
       Corresponding Source.

       1) Use a suitable shared library mechanism for linking with the
       Library.  A suitable mechanism is one that (a) uses at run time
       a copy of the Library already present on the user's computer
       system, and (b) will operate properly with a modified version
       of the Library that is interface-compatible with the Linked
       Version.

   e) Provide Installation Information, but only if you would otherwise
   be required to provide such information under section 6 of the
   GNU GPL, and only to the extent that such information is
   necessary to install and execute a modified version of the
   Combined Work produced by recombining or relinking the
   Application with a modified version of the Linked Version. (If
   you use option 4d0, the Installation Information must accompany
   the Minimal Corresponding Source and Corresponding Application
   Code. If you use option 4d1, you must provide the Installation
   Information in the manner specified by section 6 of the GNU GPL
   for conveying Corresponding Source.)

  5. Combined Libraries.

  You may place library facilities that are a work based on the
Library side by side in a single library together with other library
facilities that are not Applications and are not covered by this
License, and convey such a combined library under terms of your
choice, if you do both of the following:

   a) Accompany the combined library with a copy of the same work based
   on the Library, uncombined with any other library facilities,
   conveyed under the terms of this License.

   b) Give prominent notice with the combined library that part of it
   is a work based on the Library, and explaining where to find the
   accompanying uncombined form of the same work.

  6. Revised Versions of the GNU Lesser General Public License.

  The Free Software Foundation may publish revised and/or new versions
of the GNU Lesser General Public License from time to time. Such new
versions will be similar in spirit to the present version, but may
differ in detail to address new problems or concerns.

  Each version is given a distinguishing version number. If the
Library as you received it specifies that a certain numbered version
of the GNU Lesser General Public License "or any later version"
applies to it, you have the option of following the terms and
conditions either of that published version or of any later version
published by the Free Software Foundation. If the Library as you
received it does not specify a version number of the GNU Lesser
General Public License, you may choose any version of the GNU Lesser
General Public License ever published by the Free Software Foundation.

  If the Library as you received it specifies that a proxy can decide
whether future versions of the GNU Lesser General Public License shall
apply, that proxy's public statement of acceptance of any version is
permanent authorization for you to choose that version for the
Library.
</span></code></pre>

<h2>Concurrent-Log-Handler</h2>

<p>Concurrent Log Handler is available under Apache 2.0 license.</p>

<pre><code><span style="background-color:#f5f5f5;">                                 Apache License
                           Version 2.0, January 2004
                        http://www.apache.org/licenses/

TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION

1. Definitions.

   "License" shall mean the terms and conditions for use, reproduction,
   and distribution as defined by Sections 1 through 9 of this document.

   "Licensor" shall mean the copyright owner or entity authorized by
   the copyright owner that is granting the License.

   "Legal Entity" shall mean the union of the acting entity and all
   other entities that control, are controlled by, or are under common
   control with that entity. For the purposes of this definition,
   "control" means (i) the power, direct or indirect, to cause the
   direction or management of such entity, whether by contract or
   otherwise, or (ii) ownership of fifty percent (50%) or more of the
   outstanding shares, or (iii) beneficial ownership of such entity.

   "You" (or "Your") shall mean an individual or Legal Entity
   exercising permissions granted by this License.

   "Source" form shall mean the preferred form for making modifications,
   including but not limited to software source code, documentation
   source, and configuration files.

   "Object" form shall mean any form resulting from mechanical
   transformation or translation of a Source form, including but
   not limited to compiled object code, generated documentation,
   and conversions to other media types.

   "Work" shall mean the work of authorship, whether in Source or
   Object form, made available under the License, as indicated by a
   copyright notice that is included in or attached to the work
   (an example is provided in the Appendix below).

   "Derivative Works" shall mean any work, whether in Source or Object
   form, that is based on (or derived from) the Work and for which the
   editorial revisions, annotations, elaborations, or other modifications
   represent, as a whole, an original work of authorship. For the purposes
   of this License, Derivative Works shall not include works that remain
   separable from, or merely link (or bind by name) to the interfaces of,
   the Work and Derivative Works thereof.

   "Contribution" shall mean any work of authorship, including
   the original version of the Work and any modifications or additions
   to that Work or Derivative Works thereof, that is intentionally
   submitted to Licensor for inclusion in the Work by the copyright owner
   or by an individual or Legal Entity authorized to submit on behalf of
   the copyright owner. For the purposes of this definition, "submitted"
   means any form of electronic, verbal, or written communication sent
   to the Licensor or its representatives, including but not limited to
   communication on electronic mailing lists, source code control systems,
   and issue tracking systems that are managed by, or on behalf of, the
   Licensor for the purpose of discussing and improving the Work, but
   excluding communication that is conspicuously marked or otherwise
   designated in writing by the copyright owner as "Not a Contribution."

   "Contributor" shall mean Licensor and any individual or Legal Entity
   on behalf of whom a Contribution has been received by Licensor and
   subsequently incorporated within the Work.

2. Grant of Copyright License. Subject to the terms and conditions of
   this License, each Contributor hereby grants to You a perpetual,
   worldwide, non-exclusive, no-charge, royalty-free, irrevocable
   copyright license to reproduce, prepare Derivative Works of,
   publicly display, publicly perform, sublicense, and distribute the
   Work and such Derivative Works in Source or Object form.

3. Grant of Patent License. Subject to the terms and conditions of
   this License, each Contributor hereby grants to You a perpetual,
   worldwide, non-exclusive, no-charge, royalty-free, irrevocable
   (except as stated in this section) patent license to make, have made,
   use, offer to sell, sell, import, and otherwise transfer the Work,
   where such license applies only to those patent claims licensable
   by such Contributor that are necessarily infringed by their
   Contribution(s) alone or by combination of their Contribution(s)
   with the Work to which such Contribution(s) was submitted. If You
   institute patent litigation against any entity (including a
   cross-claim or counterclaim in a lawsuit) alleging that the Work
   or a Contribution incorporated within the Work constitutes direct
   or contributory patent infringement, then any patent licenses
   granted to You under this License for that Work shall terminate
   as of the date such litigation is filed.

4. Redistribution. You may reproduce and distribute copies of the
   Work or Derivative Works thereof in any medium, with or without
   modifications, and in Source or Object form, provided that You
   meet the following conditions:

   (a) You must give any other recipients of the Work or
   Derivative Works a copy of this License; and

   (b) You must cause any modified files to carry prominent notices
   stating that You changed the files; and

   (c) You must retain, in the Source form of any Derivative Works
   that You distribute, all copyright, patent, trademark, and
   attribution notices from the Source form of the Work,
   excluding those notices that do not pertain to any part of
   the Derivative Works; and

   (d) If the Work includes a "NOTICE" text file as part of its
   distribution, then any Derivative Works that You distribute must
   include a readable copy of the attribution notices contained
   within such NOTICE file, excluding those notices that do not
   pertain to any part of the Derivative Works, in at least one
   of the following places: within a NOTICE text file distributed
   as part of the Derivative Works; within the Source form or
   documentation, if provided along with the Derivative Works; or,
   within a display generated by the Derivative Works, if and
   wherever such third-party notices normally appear. The contents
   of the NOTICE file are for informational purposes only and
   do not modify the License. You may add Your own attribution
   notices within Derivative Works that You distribute, alongside
   or as an addendum to the NOTICE text from the Work, provided
   that such additional attribution notices cannot be construed
   as modifying the License.

   You may add Your own copyright statement to Your modifications and
   may provide additional or different license terms and conditions
   for use, reproduction, or distribution of Your modifications, or
   for any such Derivative Works as a whole, provided Your use,
   reproduction, and distribution of the Work otherwise complies with
   the conditions stated in this License.

5. Submission of Contributions. Unless You explicitly state otherwise,
   any Contribution intentionally submitted for inclusion in the Work
   by You to the Licensor shall be under the terms and conditions of
   this License, without any additional terms or conditions.
   Notwithstanding the above, nothing herein shall supersede or modify
   the terms of any separate license agreement you may have executed
   with Licensor regarding such Contributions.

6. Trademarks. This License does not grant permission to use the trade
   names, trademarks, service marks, or product names of the Licensor,
   except as required for reasonable and customary use in describing the
   origin of the Work and reproducing the content of the NOTICE file.

7. Disclaimer of Warranty. Unless required by applicable law or
   agreed to in writing, Licensor provides the Work (and each
   Contributor provides its Contributions) on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied, including, without limitation, any warranties or conditions
   of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
   PARTICULAR PURPOSE. You are solely responsible for determining the
   appropriateness of using or redistributing the Work and assume any
   risks associated with Your exercise of permissions under this License.

8. Limitation of Liability. In no event and under no legal theory,
   whether in tort (including negligence), contract, or otherwise,
   unless required by applicable law (such as deliberate and grossly
   negligent acts) or agreed to in writing, shall any Contributor be
   liable to You for damages, including any direct, indirect, special,
   incidental, or consequential damages of any character arising as a
   result of this License or out of the use or inability to use the
   Work (including but not limited to damages for loss of goodwill,
   work stoppage, computer failure or malfunction, or any and all
   other commercial damages or losses), even if such Contributor
   has been advised of the possibility of such damages.

9. Accepting Warranty or Additional Liability. While redistributing
   the Work or Derivative Works thereof, You may choose to offer,
   and charge a fee for, acceptance of support, warranty, indemnity,
   or other liability obligations and/or rights consistent with this
   License. However, in accepting such obligations, You may act only
   on Your own behalf and on Your sole responsibility, not on behalf
   of any other Contributor, and only if You agree to indemnify,
   defend, and hold each Contributor harmless for any liability
   incurred by, or claims asserted against, such Contributor by reason
   of your accepting any such warranty or additional liability.

END OF TERMS AND CONDITIONS
</span></code></pre>

<h2>PyInstaller</h2>

<pre><code><span style="background-color:#f5f5f5;">================================
     The PyInstaller licensing terms
================================


Copyright (c) 2010-2023, PyInstaller Development Team
Copyright (c) 2005-2009, Giovanni Bajo
Based on previous work under copyright (c) 2002 McMillan Enterprises, Inc.


PyInstaller is licensed under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of the License,
or (at your option) any later version.


Bootloader Exception
--------------------

In addition to the permissions in the GNU General Public License, the
authors give you unlimited permission to link or embed compiled bootloader
and related files into combinations with other programs, and to distribute
those combinations without any restriction coming from the use of those
files. (The General Public License restrictions do apply in other respects;
for example, they cover modification of the files, and distribution when
not linked into a combined executable.)


Bootloader and Related Files
----------------------------

Bootloader and related files are files which are embedded within the
final executable. This includes files in directories:

./bootloader/
./PyInstaller/loader


Run-time Hooks
----------------------------

Run-time Hooks are a different kind of files embedded within the final
executable. To ease moving them into a separate repository, or into the
respective project, these files are now licensed under the Apache License,
Version 2.0.

Run-time Hooks are in the directory
./PyInstaller/hooks/rthooks


The PyInstaller.isolated submodule
----------------------------------

By request, the PyInstaller.isolated submodule and its corresponding tests are
additionally licensed with the MIT license so that it may be reused outside of
PyInstaller under GPL 2.0 or MIT terms and conditions -- whichever is the most
suitable to the recipient downstream project. Affected files/directories are:

./PyInstaller/isolated/
./tests/unit/test_isolation.py


About the PyInstaller Development Team
--------------------------------------

The PyInstaller Development Team is the set of contributors
to the PyInstaller project. A full list with details is kept
in the documentation directory, in the file
``doc/CREDITS.rst``.

The core team that coordinates development on GitHub can be found here:
https://github.com/pyinstaller/pyinstaller.  As of 2021, it consists of:

* Hartmut Goebel
* Jasper Harrison
* Bryan Jones
* Brenainn Woodsend
* Rok Mandeljc

Our Copyright Policy
--------------------

PyInstaller uses a shared copyright model. Each contributor maintains copyright
over their contributions to PyInstaller. But, it is important to note that these
contributions are typically only changes to the repositories. Thus,
the PyInstaller source code, in its entirety is not the copyright of any single
person or institution.  Instead, it is the collective copyright of the entire
PyInstaller Development Team.  If individual contributors want to maintain
a record of what changes/contributions they have specific copyright on, they
should indicate their copyright in the commit message of the change, when they
commit the change to the PyInstaller repository.

With this in mind, the following banner should be used in any source code file
to indicate the copyright and license terms:


#-----------------------------------------------------------------------------
# Copyright (c) 2005-2023, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License (version 2
# or later) with exception for distributing the bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
#
# SPDX-License-Identifier: (GPL-2.0-or-later WITH Bootloader-exception)
#-----------------------------------------------------------------------------


For run-time hooks, the following banner should be used:

#-----------------------------------------------------------------------------
# Copyright (c) 2005-2023, PyInstaller Development Team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# The full license is in the file COPYING.txt, distributed with this software.
#
# SPDX-License-Identifier: Apache-2.0
#-----------------------------------------------------------------------------


================================
GNU General Public License
================================

https://gnu.org/licenses/gpl-2.0.html


            GNU GENERAL PUBLIC LICENSE
               Version 2, June 1991

 Copyright (C) 1989, 1991 Free Software Foundation, Inc.
 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 Everyone is permitted to copy and distribute verbatim copies
 of this license document, but changing it is not allowed.

                Preamble

  The licenses for most software are designed to take away your
freedom to share and change it.  By contrast, the GNU General Public
License is intended to guarantee your freedom to share and change free
software--to make sure the software is free for all its users.  This
General Public License applies to most of the Free Software
Foundation's software and to any other program whose authors commit to
using it.  (Some other Free Software Foundation software is covered by
the GNU Library General Public License instead.)  You can apply it to
your programs, too.

  When we speak of free software, we are referring to freedom, not
price.  Our General Public Licenses are designed to make sure that you
have the freedom to distribute copies of free software (and charge for
this service if you wish), that you receive source code or can get it
if you want it, that you can change the software or use pieces of it
in new free programs; and that you know you can do these things.

  To protect your rights, we need to make restrictions that forbid
anyone to deny you these rights or to ask you to surrender the rights.
These restrictions translate to certain responsibilities for you if you
distribute copies of the software, or if you modify it.

  For example, if you distribute copies of such a program, whether
gratis or for a fee, you must give the recipients all the rights that
you have.  You must make sure that they, too, receive or can get the
source code.  And you must show them these terms so they know their
rights.

  We protect your rights with two steps: (1) copyright the software, and
(2) offer you this license which gives you legal permission to copy,
distribute and/or modify the software.

  Also, for each author's protection and ours, we want to make certain
that everyone understands that there is no warranty for this free
software.  If the software is modified by someone else and passed on, we
want its recipients to know that what they have is not the original, so
that any problems introduced by others will not reflect on the original
authors' reputations.

  Finally, any free program is threatened constantly by software
patents.  We wish to avoid the danger that redistributors of a free
program will individually obtain patent licenses, in effect making the
program proprietary.  To prevent this, we have made it clear that any
patent must be licensed for everyone's free use or not licensed at all.

  The precise terms and conditions for copying, distribution and
modification follow.

            GNU GENERAL PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. This License applies to any program or other work which contains
a notice placed by the copyright holder saying it may be distributed
under the terms of this General Public License.  The "Program", below,
refers to any such program or work, and a "work based on the Program"
means either the Program or any derivative work under copyright law:
that is to say, a work containing the Program or a portion of it,
either verbatim or with modifications and/or translated into another
language.  (Hereinafter, translation is included without limitation in
the term "modification".)  Each licensee is addressed as "you".

Activities other than copying, distribution and modification are not
covered by this License; they are outside its scope.  The act of
running the Program is not restricted, and the output from the Program
is covered only if its contents constitute a work based on the
Program (independent of having been made by running the Program).
Whether that is true depends on what the Program does.

  1. You may copy and distribute verbatim copies of the Program's
source code as you receive it, in any medium, provided that you
conspicuously and appropriately publish on each copy an appropriate
copyright notice and disclaimer of warranty; keep intact all the
notices that refer to this License and to the absence of any warranty;
and give any other recipients of the Program a copy of this License
along with the Program.

You may charge a fee for the physical act of transferring a copy, and
you may at your option offer warranty protection in exchange for a fee.

  2. You may modify your copy or copies of the Program or any portion
of it, thus forming a work based on the Program, and copy and
distribute such modifications or work under the terms of Section 1
above, provided that you also meet all of these conditions:

    a) You must cause the modified files to carry prominent notices
    stating that you changed the files and the date of any change.

    b) You must cause any work that you distribute or publish, that in
    whole or in part contains or is derived from the Program or any
    part thereof, to be licensed as a whole at no charge to all third
    parties under the terms of this License.

    c) If the modified program normally reads commands interactively
    when run, you must cause it, when started running for such
    interactive use in the most ordinary way, to print or display an
    announcement including an appropriate copyright notice and a
    notice that there is no warranty (or else, saying that you provide
    a warranty) and that users may redistribute the program under
    these conditions, and telling the user how to view a copy of this
    License.  (Exception: if the Program itself is interactive but
    does not normally print such an announcement, your work based on
    the Program is not required to print an announcement.)

These requirements apply to the modified work as a whole.  If
identifiable sections of that work are not derived from the Program,
and can be reasonably considered independent and separate works in
themselves, then this License, and its terms, do not apply to those
sections when you distribute them as separate works.  But when you
distribute the same sections as part of a whole which is a work based
on the Program, the distribution of the whole must be on the terms of
this License, whose permissions for other licensees extend to the
entire whole, and thus to each and every part regardless of who wrote it.

Thus, it is not the intent of this section to claim rights or contest
your rights to work written entirely by you; rather, the intent is to
exercise the right to control the distribution of derivative or
collective works based on the Program.

In addition, mere aggregation of another work not based on the Program
with the Program (or with a work based on the Program) on a volume of
a storage or distribution medium does not bring the other work under
the scope of this License.

  3. You may copy and distribute the Program (or a work based on it,
under Section 2) in object code or executable form under the terms of
Sections 1 and 2 above provided that you also do one of the following:

    a) Accompany it with the complete corresponding machine-readable
    source code, which must be distributed under the terms of Sections
    1 and 2 above on a medium customarily used for software interchange; or,

    b) Accompany it with a written offer, valid for at least three
    years, to give any third party, for a charge no more than your
    cost of physically performing source distribution, a complete
    machine-readable copy of the corresponding source code, to be
    distributed under the terms of Sections 1 and 2 above on a medium
    customarily used for software interchange; or,

    c) Accompany it with the information you received as to the offer
    to distribute corresponding source code.  (This alternative is
    allowed only for noncommercial distribution and only if you
    received the program in object code or executable form with such
    an offer, in accord with Subsection b above.)

The source code for a work means the preferred form of the work for
making modifications to it.  For an executable work, complete source
code means all the source code for all modules it contains, plus any
associated interface definition files, plus the scripts used to
control compilation and installation of the executable.  However, as a
special exception, the source code distributed need not include
anything that is normally distributed (in either source or binary
form) with the major components (compiler, kernel, and so on) of the
operating system on which the executable runs, unless that component
itself accompanies the executable.

If distribution of executable or object code is made by offering
access to copy from a designated place, then offering equivalent
access to copy the source code from the same place counts as
distribution of the source code, even though third parties are not
compelled to copy the source along with the object code.

  4. You may not copy, modify, sublicense, or distribute the Program
except as expressly provided under this License.  Any attempt
otherwise to copy, modify, sublicense or distribute the Program is
void, and will automatically terminate your rights under this License.
However, parties who have received copies, or rights, from you under
this License will not have their licenses terminated so long as such
parties remain in full compliance.

  5. You are not required to accept this License, since you have not
signed it.  However, nothing else grants you permission to modify or
distribute the Program or its derivative works.  These actions are
prohibited by law if you do not accept this License.  Therefore, by
modifying or distributing the Program (or any work based on the
Program), you indicate your acceptance of this License to do so, and
all its terms and conditions for copying, distributing or modifying
the Program or works based on it.

  6. Each time you redistribute the Program (or any work based on the
Program), the recipient automatically receives a license from the
original licensor to copy, distribute or modify the Program subject to
these terms and conditions.  You may not impose any further
restrictions on the recipients' exercise of the rights granted herein.
You are not responsible for enforcing compliance by third parties to
this License.

  7. If, as a consequence of a court judgment or allegation of patent
infringement or for any other reason (not limited to patent issues),
conditions are imposed on you (whether by court order, agreement or
otherwise) that contradict the conditions of this License, they do not
excuse you from the conditions of this License.  If you cannot
distribute so as to satisfy simultaneously your obligations under this
License and any other pertinent obligations, then as a consequence you
may not distribute the Program at all.  For example, if a patent
license would not permit royalty-free redistribution of the Program by
all those who receive copies directly or indirectly through you, then
the only way you could satisfy both it and this License would be to
refrain entirely from distribution of the Program.

If any portion of this section is held invalid or unenforceable under
any particular circumstance, the balance of the section is intended to
apply and the section as a whole is intended to apply in other
circumstances.

It is not the purpose of this section to induce you to infringe any
patents or other property right claims or to contest validity of any
such claims; this section has the sole purpose of protecting the
integrity of the free software distribution system, which is
implemented by public license practices.  Many people have made
generous contributions to the wide range of software distributed
through that system in reliance on consistent application of that
system; it is up to the author/donor to decide if he or she is willing
to distribute software through any other system and a licensee cannot
impose that choice.

This section is intended to make thoroughly clear what is believed to
be a consequence of the rest of this License.

  8. If the distribution and/or use of the Program is restricted in
certain countries either by patents or by copyrighted interfaces, the
original copyright holder who places the Program under this License
may add an explicit geographical distribution limitation excluding
those countries, so that distribution is permitted only in or among
countries not thus excluded.  In such case, this License incorporates
the limitation as if written in the body of this License.

  9. The Free Software Foundation may publish revised and/or new versions
of the General Public License from time to time.  Such new versions will
be similar in spirit to the present version, but may differ in detail to
address new problems or concerns.

Each version is given a distinguishing version number.  If the Program
specifies a version number of this License which applies to it and "any
later version", you have the option of following the terms and conditions
either of that version or of any later version published by the Free
Software Foundation.  If the Program does not specify a version number of
this License, you may choose any version ever published by the Free Software
Foundation.

  10. If you wish to incorporate parts of the Program into other free
programs whose distribution conditions are different, write to the author
to ask for permission.  For software which is copyrighted by the Free
Software Foundation, write to the Free Software Foundation; we sometimes
make exceptions for this.  Our decision will be guided by the two goals
of preserving the free status of all derivatives of our free software and
of promoting the sharing and reuse of software generally.

                NO WARRANTY

  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
REPAIR OR CORRECTION.

  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

             END OF TERMS AND CONDITIONS

================================
Apache License 2.0
================================

                                 Apache License
                           Version 2.0, January 2004
                        http://www.apache.org/licenses/

   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION

   1. Definitions.

      "License" shall mean the terms and conditions for use, reproduction,
      and distribution as defined by Sections 1 through 9 of this document.

      "Licensor" shall mean the copyright owner or entity authorized by
      the copyright owner that is granting the License.

      "Legal Entity" shall mean the union of the acting entity and all
      other entities that control, are controlled by, or are under common
      control with that entity. For the purposes of this definition,
      "control" means (i) the power, direct or indirect, to cause the
      direction or management of such entity, whether by contract or
      otherwise, or (ii) ownership of fifty percent (50%) or more of the
      outstanding shares, or (iii) beneficial ownership of such entity.

      "You" (or "Your") shall mean an individual or Legal Entity
      exercising permissions granted by this License.

      "Source" form shall mean the preferred form for making modifications,
      including but not limited to software source code, documentation
      source, and configuration files.

      "Object" form shall mean any form resulting from mechanical
      transformation or translation of a Source form, including but
      not limited to compiled object code, generated documentation,
      and conversions to other media types.

      "Work" shall mean the work of authorship, whether in Source or
      Object form, made available under the License, as indicated by a
      copyright notice that is included in or attached to the work
      (an example is provided in the Appendix below).

      "Derivative Works" shall mean any work, whether in Source or Object
      form, that is based on (or derived from) the Work and for which the
      editorial revisions, annotations, elaborations, or other modifications
      represent, as a whole, an original work of authorship. For the purposes
      of this License, Derivative Works shall not include works that remain
      separable from, or merely link (or bind by name) to the interfaces of,
      the Work and Derivative Works thereof.

      "Contribution" shall mean any work of authorship, including
      the original version of the Work and any modifications or additions
      to that Work or Derivative Works thereof, that is intentionally
      submitted to Licensor for inclusion in the Work by the copyright owner
      or by an individual or Legal Entity authorized to submit on behalf of
      the copyright owner. For the purposes of this definition, "submitted"
      means any form of electronic, verbal, or written communication sent
      to the Licensor or its representatives, including but not limited to
      communication on electronic mailing lists, source code control systems,
      and issue tracking systems that are managed by, or on behalf of, the
      Licensor for the purpose of discussing and improving the Work, but
      excluding communication that is conspicuously marked or otherwise
      designated in writing by the copyright owner as "Not a Contribution."

      "Contributor" shall mean Licensor and any individual or Legal Entity
      on behalf of whom a Contribution has been received by Licensor and
      subsequently incorporated within the Work.

   2. Grant of Copyright License. Subject to the terms and conditions of
      this License, each Contributor hereby grants to You a perpetual,
      worldwide, non-exclusive, no-charge, royalty-free, irrevocable
      copyright license to reproduce, prepare Derivative Works of,
      publicly display, publicly perform, sublicense, and distribute the
      Work and such Derivative Works in Source or Object form.

   3. Grant of Patent License. Subject to the terms and conditions of
      this License, each Contributor hereby grants to You a perpetual,
      worldwide, non-exclusive, no-charge, royalty-free, irrevocable
      (except as stated in this section) patent license to make, have made,
      use, offer to sell, sell, import, and otherwise transfer the Work,
      where such license applies only to those patent claims licensable
      by such Contributor that are necessarily infringed by their
      Contribution(s) alone or by combination of their Contribution(s)
      with the Work to which such Contribution(s) was submitted. If You
      institute patent litigation against any entity (including a
      cross-claim or counterclaim in a lawsuit) alleging that the Work
      or a Contribution incorporated within the Work constitutes direct
      or contributory patent infringement, then any patent licenses
      granted to You under this License for that Work shall terminate
      as of the date such litigation is filed.

   4. Redistribution. You may reproduce and distribute copies of the
      Work or Derivative Works thereof in any medium, with or without
      modifications, and in Source or Object form, provided that You
      meet the following conditions:

      (a) You must give any other recipients of the Work or
          Derivative Works a copy of this License; and

      (b) You must cause any modified files to carry prominent notices
          stating that You changed the files; and

      (c) You must retain, in the Source form of any Derivative Works
          that You distribute, all copyright, patent, trademark, and
          attribution notices from the Source form of the Work,
          excluding those notices that do not pertain to any part of
          the Derivative Works; and

      (d) If the Work includes a "NOTICE" text file as part of its
          distribution, then any Derivative Works that You distribute must
          include a readable copy of the attribution notices contained
          within such NOTICE file, excluding those notices that do not
          pertain to any part of the Derivative Works, in at least one
          of the following places: within a NOTICE text file distributed
          as part of the Derivative Works; within the Source form or
          documentation, if provided along with the Derivative Works; or,
          within a display generated by the Derivative Works, if and
          wherever such third-party notices normally appear. The contents
          of the NOTICE file are for informational purposes only and
          do not modify the License. You may add Your own attribution
          notices within Derivative Works that You distribute, alongside
          or as an addendum to the NOTICE text from the Work, provided
          that such additional attribution notices cannot be construed
          as modifying the License.

      You may add Your own copyright statement to Your modifications and
      may provide additional or different license terms and conditions
      for use, reproduction, or distribution of Your modifications, or
      for any such Derivative Works as a whole, provided Your use,
      reproduction, and distribution of the Work otherwise complies with
      the conditions stated in this License.

   5. Submission of Contributions. Unless You explicitly state otherwise,
      any Contribution intentionally submitted for inclusion in the Work
      by You to the Licensor shall be under the terms and conditions of
      this License, without any additional terms or conditions.
      Notwithstanding the above, nothing herein shall supersede or modify
      the terms of any separate license agreement you may have executed
      with Licensor regarding such Contributions.

   6. Trademarks. This License does not grant permission to use the trade
      names, trademarks, service marks, or product names of the Licensor,
      except as required for reasonable and customary use in describing the
      origin of the Work and reproducing the content of the NOTICE file.

   7. Disclaimer of Warranty. Unless required by applicable law or
      agreed to in writing, Licensor provides the Work (and each
      Contributor provides its Contributions) on an "AS IS" BASIS,
      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
      implied, including, without limitation, any warranties or conditions
      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
      PARTICULAR PURPOSE. You are solely responsible for determining the
      appropriateness of using or redistributing the Work and assume any
      risks associated with Your exercise of permissions under this License.

   8. Limitation of Liability. In no event and under no legal theory,
      whether in tort (including negligence), contract, or otherwise,
      unless required by applicable law (such as deliberate and grossly
      negligent acts) or agreed to in writing, shall any Contributor be
      liable to You for damages, including any direct, indirect, special,
      incidental, or consequential damages of any character arising as a
      result of this License or out of the use or inability to use the
      Work (including but not limited to damages for loss of goodwill,
      work stoppage, computer failure or malfunction, or any and all
      other commercial damages or losses), even if such Contributor
      has been advised of the possibility of such damages.

   9. Accepting Warranty or Additional Liability. While redistributing
      the Work or Derivative Works thereof, You may choose to offer,
      and charge a fee for, acceptance of support, warranty, indemnity,
      or other liability obligations and/or rights consistent with this
      License. However, in accepting such obligations, You may act only
      on Your own behalf and on Your sole responsibility, not on behalf
      of any other Contributor, and only if You agree to indemnify,
      defend, and hold each Contributor harmless for any liability
      incurred by, or claims asserted against, such Contributor by reason
      of your accepting any such warranty or additional liability.

   END OF TERMS AND CONDITIONS

   APPENDIX: How to apply the Apache License to your work.

      To apply the Apache License to your work, attach the following
      boilerplate notice, with the fields enclosed by brackets "[]"
      replaced with your own identifying information. (Don't include
      the brackets!)  The text should be enclosed in the appropriate
      comment syntax for the file format. We also recommend that a
      file or class name and description of purpose be included on the
      same "printed page" as the copyright notice for easier
      identification within third-party archives.

   Copyright [yyyy] [name of copyright owner]

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

===========
MIT License
===========

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
</span></code></pre>

<h2>Icons8</h2>

<p>Icons from the Icons8 website are provided under the license for a Free Account Holder.</p>

<pre><code><span style="background-color:#f5f5f5;">Using, downloading, or copying images or content from our website or that of our
subsidiaries constitutes your acceptance of these Licensing Terms as well as any
changes to this Agreement. Please revisit this Agreement prior to purchasing any future content licenses in order to be aware of changes.

Definitions


For purposes of this Licensing Agreement, the following terms and definitions apply:

    Breach: A violation of this Agreement or any of our Company's Agreements or Policies.

    CISG: U.N. Convention for the International Sale of Goods (C.I.S.G.) which provides for the use of arbitration in order to resolve international business disputes.

    Dispute: Any dispute or disagreement regarding Licensing Agreement terms or their application.

    DMCA: Digital Millennium Copyright Act: A U.S.A. copyright law which implements world WIPO treaties protecting copyrighted works.

    Exclusive License: An Exclusive Arrangement with Illustrators (Icons8 and Ouch) when Icons8 purchases the rights to sell Illustrator content on Company website(s).

    Free Account Holder: An individual User or Licensee accessing and using the Icons8 or subsidiary websites for personal use without having paid a licensing fee. This includes users who have not registered or logged in. Certain restrictions apply such as providing links and references for usage.

    Governing Law: The laws, rules, and regulations governing the application and

    interpretation of this Licensing Agreement.

    Intellectual Property: Any work, intangible work, creation, design, symbol, mark,

    trademark, logo, and/or image that is used in commerce including but not limited to: illustrations, icons, symbols, photographs, or music.

    Licensed Material: Also "Content", "Licensed Content", or "Licensed Material",

    including but not limited to: icons; illustrations; graphics; photographs; and/or music.

    Method of license acquisition including but not limited to: subscription, single purchase, and/or time-based contract does not effect change on Licensed Material.

    Licensee: User or purchaser of content license, any person, entity, or any authorized representative of the company, person, or third-party, that uses, downloads, or otherwise accesses Licensed Materials from our website or that of our subsidiaries; also "you" or "your".

    Licensor: Icons8 or a duly authorized representative. Also, "Company", "We", "Us", or "Our".

    Mediation: Informal, third-party conflict resolution as opposed to going to court or

    using arbitration to resolve a disagreement.

    Subsidiary: Subordinate company, entity, or website owned or controlled by Icons8.

    User: Account holder, Free account holder, Licensee, purchaser of Licensed Content, or their authorized representative.

    Universal Multimedia Licensing Agreement ("Agreement", or "License"): Agreement between Icons8 and the Licensee granting permission to use Licensed Materials and content with certain provisions or restrictions.

License Usage

a. A "License" or consent is issued based on the number of individual users who have access to the Licensed Materials. A single License is issued to persons that are Free account holders, individuals, or companies in the event there is only one (1) person accessing and/or using Licensed Material. Note: It is strictly prohibited for more than one (1) User or Licensee to use or access a single account at any time.

b. The Licensee is required under the terms of this Agreement to insert the icons or
   multimedia into their own personal artwork, project, or software application.

c. If a Licensee or User shall have two or more persons using or accessing an Icons8
account or subsidiary account at any time, a team account or an "Enterprise" License is required. Please 
contact Icons8 Customer Service at https://icons8.com/contact for information on how to purchase multiple licenses or get an "Enterprise" License.

d. The Licensee's use of any Licensed Material in a manner not permitted under the terms of this Agreement constitutes copyright infringement, and Icons8 shall be entitled to exercise any and all rights or remedies in order to enforce these terms. The Licensee understands and agrees to be liable and responsible for any and all damages resulting from the said infringement, including but not limited to legal and attorney fees.

e. The Licensee is responsible for any and all appropriate sales or uses tax imposed by any government authority or jurisdiction as a result of the License granted to the


Licensee by Icons8.

f. The Licensee may not sell or otherwise distribute the final product made solely or majorly of Icons8 icons, photos, illustrations, or music without the written consent of Icons8. Please contact Icons8 Customer Service at https://icons8.com/contact for information if you'd like to sell or distribute our products.
Image License

a. Icons8 and its' subsidiaries grant the User or Licensee a worldwide, non-exclusive,
non-transferable right to use; modify; or reproduce images; icons; photographs; and
illustrations subject to restrictions or limitations as provided within this Licensing


Agreement.

b. Free and paid account holders may provide the third parties with the final product
containing the Licensed Materials if the third party agrees to the same terms and
conditions (this document) as the Licensee. It is the Licensee's responsibility to
include a copy of the terms and conditions (this document) when providing third
parties with the final product. Licensees are obligated to make third parties aware of
the rules for crediting and distributing Licensed Materials stated in previous
paragraphs.

There is no restriction on personal or commercial use of any kind, providing:

I. Free Account Holders

    Free Account Holders must not make derivative copies of any icons or music
    accessed, used, or downloaded from our website; however, Free Account Holders
    may use Icons8 Tools (Mega Creator, Iconizer, OMGIMG, Pichon, Lunacy) to modify or change illustrations or photographs.
    Free Account Holders must provide two hyperlinks to Licensed Materials used or
    embedded in a website or social media site as generated by the Icons8 website (or subsidiary site). One link shall be to the Icons8 website, and a secondary link to the location of the downloaded work, icon, photograph, illustration, or music. To
    maintain Free Account Holder status usage must comply with the terms of this Agreement.

II.Paid Users/Licensees

    All paid Users and Licensees may make changes to photographs using our Mega
    Creator: https://icons8.com/mega-creator, including but not limited to derivative
    copies.

    All paid Users and Licensees may make changes to or derivative copies of all
    Licensed Material found on the Icons8 website or subsidiary website, including but
    not limited to: icons; illustrations; photographs; and music.

    A User or Licensee may not resell or distribute Licensed Materials as stand-alone
    files (Example: Separate icon files in SVG format) without the written consent of Icons8.

    A User or Licensee is prohibited from exporting, distributing, or otherwise sharing our Licensed Content and Materials as stand-alone files (separating, reverse engineering, or making derivative copies of our Licensed Materials).


b. Images and Licensed Materials may be used in any manner without restriction as to
the number of copies for personal or commercial usage provided such usage complies with provisions within this Agreement.

c. The Licensee acknowledges that Icons8 retains all rights, title, and interest in
copyrighted illustrations, icons, photographs, music, or other media created by Icons8
staff; employees; or agents. Independent artists that create icons; photographs;
music; or other Licensed Material for sale or distribution on the Icons8 website retain
full rights of ownership and interest in their original works as provided herein.


Exclusive Illustrator License

Illustrators creating licensed material and content for https://icons8.com/illustrations understand and agree that they are engaging in an exclusive licensing arrangement with Icons8 and for illustrations and works created and uploaded to the Icons8 website for promotion or distribution. Our Company compensates illustrators for their work with a one-time royalty payment. After our Company purchases the illustration, the Illustrator acknowledges that:

    They are not entitled to additional royalties;

    Icons8 and its subsidiaries own any rights, title, and interest in all illustrations uploaded by him/her to the Icons8.com websites.


Music License

I. Musicians. Musicians and artists grant Icons8 a worldwide, unlimited, transferable,
perpetual license and consent to use, display, play, market, advertise, or otherwise
promote their music and works for use by our Customers, Users, and Licensees, in
exchange for monthly royalty payments.

II. Licensees. Licensees and Users are granted a restricted license to use, change, or
modify any or all recorded works displayed or uploaded on our website with the
exception of Free Account Holders. Use of Licensed Material or Content is subject to
restrictions noted within this Agreement.

III.Commercial Use. Licensed Material or Content may not be: exported, shared,
downloaded or uploaded, or otherwise displayed specifically for commercial use
(example: As a Re-seller), nor shall works be distributed, shared, exported, up or
downloaded as a stand-alone file.

IV. Content ID. Due to administration costs, music marked with Content ID is not licensed for YouTube. However, users are welcome to negotiate the terms with the musicians directly.

V. Editing. Permitted uses for Free Account Holders: synchronization with an audio-visual or audio-only work (allowed variations: cutting, stretching, looping). Prohibited Uses for Free Account Holders: remixing, mashing up, adding lyrics.

Permitted License Usage

A Licensee may, subject to License usage, permission, and/or restrictions noted within
this Agreement, use, alter, crop, modify, adapt, or make derivative works of the Licensed Materials found on our website or subsidiary website.

Permitted License Usage includes:

    Use of Licensed Material for the Internet or Intranet;

    Use of Licensed Material for Desktop or Mobile Applications;

    Using Licensed Material to create Web Templates for Sale;

    Use of Licensed Material in WordPress themes for re-sale, provided the Content is not intended for redistribution or re-use of the Material. The number of Licensed Material items used in a WordPress theme can not exceed 200. All WordPress themes containing Licensed Material must include the following reference: https://icons8.com.

    Using Licensed Material for Print Media (provided the Licensee is not sub-licensing or allowing redistribution or re-use of the Licensed Material);

    Use of artwork or Licensed Material for re-sale, provided the Content is not intended for redistribution or re-use of the Material;

    Use of Licensed Material for the purpose of creating a logo, trademark, or any other kind of visual representation (providing that the licensee is not officially registering it with a patent office).

Restrictions

This Universal Multimedia License carries the following prohibitions or restrictions.

The Licensee may not:

    Sublicense, distribute, sell, assign, convey, or otherwise transfer this license nor any of the Licensee's rights, duties, or legal responsibilities under this Agreement;

    Modify, alter, or change any of the Terms of this Universal Multimedia Licensing

    Agreement without the express written consent of Icons8 or its' authorized

    representatives;

    Sell, resell, license, or distribute its' final product in a manner allowing others or third parties to extract or access Licensed Material as stand-alone files;

    Distribute, export, post, upload, or download Licensed Material online in a

    the downloadable format or enable Licensed Material to be accessed; downloaded, or distributed in any manner that would be prohibited by this Agreement;

    Use Licensed Material in any manner that infringes upon any third-party copyrights, trade names, trademarks, or service marks;

    Use Licensed Material within a work product for the purpose of creating a final product wherein Licensed Materials are going to be used (i.e., graphic editors, rapid prototyping tools, and programming IDEs). This does not include WordPress themes;

    Use Licensed Material for the purposes of training machine learning or artificial

    intelligence systems;

    Use of Licensed Material for Broadcasts and Theatrical Productions or Exhibitions;

    Officially register the Licensed Material or any part of it as their or their company's

    logo, trademark, or any other visual representation;

    Ship, transport, export, or share Licensed Material with a third-party in any country or territory, or use Material in any manner prohibited by export administration legislation as well as any other export laws, regulations, or restrictions.

DMCA Policy

If a Licensee or User has committed a copyright violation, please notify Customer Service immediately at: copyright@icons8.com.

a. Claims. A copyright violation Claim (i.e., "Claim") must be in writing and sent to our
Company via email or the U.S. Postal Service. The Claim must contain the
Claimants' (i.e., Person Making a Claim) name, address, email address, contact
information, and proof of intellectual property ownership or rights to ownership. The

Claim must also:

    Identify the property, work, or content that has allegedly been infringed;

    Identify the location of the property, work, or content Claimant believes to have

    been allegedly infringed;

    State the name or identity, as well as contact information of the person or

    persons(s) the Claimant, believes committed the alleged infringement;

    Provide a signed statement under penalty of perjury, stating the Claimant did not

    give permission to the alleged violator to use copyrighted material;

    If the Claimant is not the owner of original content, then he/she will also need to

    supply our Company with a signed statement verifying they are authorized to

    represent the original owner of the content, work, or property, and provide the original owner's contact information.

b. Investigation. Our Company will conduct an investigation when our Copyright Agent
receives the Claim. We will immediately take the following actions: (I) Freeze the
alleged violator's account; (II) Conduct an investigation; (III) Verify the Claim; (IV)
Notify the violator of the Claim; and (V) Send the Claimant a prompt response about
our investigation.

c. Warnings and False Claims. First-time copyright offenders will receive a written
warning, and their content will be deleted. Upon receiving a second complaint, the
offender's account will be terminated. If our Copyright Agent discovers the Claimant
filed a false claim, we will immediately release any disputed material back to the
original owner and unfreeze their account.

d. Responsibility. The Claimant also agrees to be personally and financially responsible for all costs or fees incurred as a result of filing a false claim, as well as the cost of the investigation, including but not limited to attorney's fees, legal fees, and expenses.


License Expiration

a. The term of this Universal Multimedia License begins immediately upon the Licensee's access, acceptance, download, upload, or purchase of any Licensed Material from the Icons8 or subsidiary website, and is valid for the term specified within the Licensee or User's particular licensing package agreed upon at purchase (i.e., "Licensing Period").

This License covers the following aspects of the Licensed Material:

    New Features, fixes, and improvements to the software, media, and recorded

    content;

    Licensed Material usage and restrictions;

    New Licensed Material and Content;

    Email Support.

b. After expiration and termination of this Universal Multimedia Licensing Agreement, the following features shall remain and be applicable to:

    Licensed Materials created before expiration of the Agreement;

    Licensee's use of our website (or subsidiary websites), desktop software, or

    multimedia content with the Licensed Materials created prior to expiration;

c. Upon expiration, this License and Agreement shall not apply to:

    Newly purchased Licensed Materials;

    Customer Support.

Licensee Warranty

The Licensee or User warrants and agrees to be responsible for all Licensed Materials
uploaded, downloaded, or otherwise transmitted between our website and the end User.

Under the terms of this Universal Multimedia Agreement, the Licensee shall not permit
the unauthorized display, access, distribution, use, or sharing of any Licensed Material or Content. The Licensee also agrees:

    To Indemnify and hold Icons8 (and its' subsidiaries, owners, directors, employees, and agents) harmless from any and all: actions, proceedings, claims, unforeseen third-party claims, demands, costs (including without restriction any legal costs, fees or expenses), awards, and damages arising directly or indirectly from this Agreement, or as a result of any breach or non-performance of this Agreement by the Licensee.

    Not to assign, transfer, or sublicense this Agreement, or permit the unauthorized use of Licensed Materials or any portion thereof for any purpose;

    Not to assign or transfer its duties or obligations under this Agreement, nor to loan, rent, export, or transfer any unauthorized licensing rights or privileges to any other person or entity;

    That Licensed Materials shall not infringe upon any third-party rights whatsoever, and any liability for the infringement of third-party rights including but not limited to damages and legal costs shall rest with the Licensee.

Licensor Warranty

a. Infringement. Pursuant to the terms of this Agreement and the purchased Universal

Multimedia License, Icons8 (and its' subsidiaries) warrant that the Licensed Material
will not infringe upon any copyrights or intellectual property rights of any third-party
provided the Licensee uses the Content pursuant to the terms of this Agreement.

b. "As Is". The User or Licensee understands and agrees that any and all Licensed

Material is provided "as is" without representation, warranty, or condition of any kind
either express or implied, including but not limited to implied warranties,
representations, or conditions of merchantability, or fitness for a particular purpose.

Icons8 (and its' subsidiaries) do not warrant or guarantee that the Licensed Content
will meet the Licensee or User's requirements or that their use will be uninterrupted or
error-free. The Licensee acknowledges and agrees to bear all risk for the use of the

Licensed Material and Content.

c. Defective Content. Should the uploaded, downloaded, or transmitted Licensed

Material or Content prove to be defective, the Licensee agrees to assume all risk and
cost for corrections. Our Company makes no representation or warranty respecting

Licensed Material that is downloaded in a flash format or file except as expressly
provided herein or within our agreements and policies (as it applies to ownership,
technical requirements, or legal compliance).

Intellectual Property

a. Rights. Licensed Material and all icons are owned by Icons8 which are protected by

U.S. and International Copyright Laws, rules, and regulations. No title or intellectual
property rights related to the Licensed Material are transferred or assigned to the

Licensee. The Licensor retains all rights, title, and interest not expressly granted by
this License Agreement.

b. Trademarks. Some of the Licensed Material may contain copyrighted content such as trademarks and/or movie characters. These icons shall be used in accordance with
accepted trademark practice, as well as a professional standard of care. Users or

Licensees may be required to identify a trademark owner's name, under the
provisions of this Agreement. Trademarks may only be used to identify printed or
electronic material produced by the icons. You understand and agree that use of any
trademark or service mark does not give you any rights of ownership in that
trademark. Except as otherwise provided herein, this License Agreement does not
grant the User or Licensee any intellectual property rights, nor rights to royalties to

Licensed Materials.

Limited Liability

a. Liability. At no time shall any of our companies, owners, directors, officers, employees, representatives, affiliates, or content providers; be liable for any incidental, indirect, punitive, exemplary, or consequential damages (including but not limited to loss of profits, interruption, loss of business information or any other pecuniary loss) in connection with any claim, loss, damage, or other proceeding arising under the terms of this Agreement, including without limitation your use of, reliance upon, access to, or exploitation of the Licensed Content, or any part thereof, or any rights granted to the Licensee or User as provided herein, even if our Company had been advised of the possibility of such damages, whether the action is based upon contract, tort (including negligence), infringement of intellectual property rights or otherwise. No action, regardless of form or nature, arising out of this Agreement may be brought by or on behalf of the Claimant more than one (1) year after the cause of action first arose. Our companies, owners, directors, employees, affiliates, and/or representatives shall not be liable at any time for damages, costs, or losses arising as a result of modifications made to the Licensed Material or Content.

b. Limitation of Liability. Notwithstanding any other terms under this Agreement, the
total maximum aggregate liability under this Agreement and any other agreement
where our Company has licensed the same Material or Content, regardless of file
size, or the Licensee/User's exploitation of any or all Licensed Content in any manner,
our aggregate liability shall be limited to Ten ($10.00) U.S. Dollars per event. Some
jurisdictions do not allow the limitation or exclusion of liability for incidental or
consequential damages; therefore, the limitations noted herein may not apply to you
specifically.

Force Majeure

All parties to this Agreement acknowledge and agree that neither shall be responsible for any delay or failure to perform under these terms in the event any delay or failure is caused by fire, flood, strike, labor unrest, riot, embargo, act of civil or military authority, accident, acts of God, vandalism, or other event beyond its' control. Notice of any Force Majeure Event shall be given to the non-affected party as soon as possible after its occurrence. The affected party(ies) shall resume their duties and responsibilities as soon as it is practical.

Termination And Breach Of Agreement

Violations of any terms contained within this Universal Multimedia Agreement shall be
considered a "breach" or infraction. In the event of breach, the Licensor shall provide the Licensee with written notice of the infraction. Upon receipt of notice, the Licensee shall have seven (7) days to cure the infraction or violation. If the Licensee fails to timely cure the breach, then this Agreement is deemed to have been automatically terminated without any further notice to the Licensee. The Licensee agrees to be financially and personally responsible for such breach including but not limited to: damages, fees, or balances on account.

Disputes

a. Mediation. In the event of a dispute regarding this Agreement, it's terms or their
interpretation, it is agreed that the Licensor, Licensee, or User, will attempt to resolve
the dispute in a professional manner. If the parties are unable to resolve the dispute
personally, then they both agree to use the services of a Private Mediator in
Delaware, U.S.A. The parties agree to share the cost and choice of a Mediator.

b. Notice of Intent. The Disputing Party shall send the Non-Disputing party a thirty (30)
day written "Notice of Intent" stating the basis for the intent to Mediate prior to
actually filing a Claim. Service shall be completed by delivery of the Notice to the
Non-Disputing Party's last known address.

c. Procedure. The Mediation Hearing shall be conducted via Mediation or Arbitration

Rules as adopted by the American Bar Association.

d. Award. The prevailing party shall be entitled to reimbursement of any costs and fees, including but not limited to attorney's fees. This Mediation provision shall survive
cancellation or termination of this Agreement.
Governing Law

This Agreement shall be governed by the laws and statutes of the State of Delaware,

United States of America, excluding the application of conflict of law provisions. This

Agreement shall not be governed by the United Nations Convention on Contracts for the International Sale of Goods (C.I.S.G.), the application of which is expressly excluded.

General Provisions

If any part of this Agreement is found by a court of competent jurisdiction or mediator to be void and unenforceable, the void or unenforceable provision shall not affect the
validity or interpretation of the balance of the Agreement, which shall remain valid and
enforceable according to its terms.

Contact

If you have questions or comments regarding this Universal Multimedia Licensing

Agreement, please contact us at:
https://icons8.com/contact
support@icons8.com
Tel: + 1 929-445-2737
</span></code></pre>

<h2>Icons</h2>

<p>The icons from SublimeMerge, TortoiseGit, Git, SourceTree remain the property of their respective owners and
are copyrighted to their respective owner. They are used in Multigit for information purpose, 
under the "Fair use" law.</p>

<h2>Git logo</h2>

<p>Multigit logo is derived from the official Git Logo, by Jason Long. The Git logo is licensed under the Creative Commons 
Attribution 3.0 Unported License.</p>

<p>See <a href="https://git-scm.com/downloads/logos">Git logos</a></p>

<pre><code><span style="background-color:#f5f5f5;">Attribution 3.0 Unported (CC BY 3.0)

This is a human-readable summary of (and not a substitute for) the license. Disclaimer.
You are free to:

    Share - copy and redistribute the material in any medium or format
    Adapt - remix, transform, and build upon the material
    for any purpose, even commercially.


This license is acceptable for Free Cultural Works.

    The licensor cannot revoke these freedoms as long as you follow the license terms.



Under the following terms:

    Attribution - You must give appropriate credit, provide a link to the license, and indicate if changes 
    were made. You may do so in any reasonable manner, but not in any way that suggests the licensor endorses you or your use.

    No additional restrictions - You may not apply legal terms or technological measures that legally restrict others from 
    doing anything the license permits.

Notices:

    You do not have to comply with the license for elements of the material in the public domain or where your use is 
    permitted by an applicable exception or limitation.

    No warranties are given. The license may not give you all of the permissions necessary for your intended use. For 
    example, other rights such as publicity, privacy, or moral rights may limit how you use the material.
</span></code></pre>

'''
